//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2002                 www.xilinx.com
//
//  XAPP 228 - Quad port RAM in Virtex devices
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :       top_v2.v
//
//  Description :     Example top-level for quad-port memory in VirtexII
//
//  Date - revision : June 4th 2003 - v 1.1
//
//  Author :          NJS
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors makeand you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specificallydisclaims any
//              implied warranties of merchantability, non-infringement,or
//              fitness for a particular purpose. Xilinx does notwarrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designswill be
//              uninterrupted or error free, or that defects in theDesigns
//              will be corrected. Furthermore, Xilinx does not warrantor
//              make any representations regarding use or the results ofthe
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits,cost
//              or procurement of substitute goods or services, or forany
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on anytheory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstanding the failure ofthe
//              essential purpose of any limited remedies herein.
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved
//
///////////////////////////////////////////////////////////////////////////////
//
`timescale 1 ps / 1ps

module top_v2 (
	rstin,			// reset input (active low)
	clkacin,		// clock for ports a and c
	clkbdin,		// clock for ports b and d (faster in this example)
	matchab,		// A to B transfer working
	matchcd,		// C to D transfer working
	locked) ;		// DCMs locked

input 		rstin, clkacin, clkbdin ;

output 		matchab, matchcd, locked ;
reg 		matchab, matchcd ;

wire  		high ;
wire		low ;
wire		clkacint ;
wire		clkbdint ;
wire		clkacx2 ;
wire		clkbdx2 ;
wire		clkacdcm ;
wire		clkbddcm ;
wire		clkacx2dcm ;
wire		clkbdx2dcm ;
wire		clkac ;
wire		clkbd ;
wire		locked_ac ;
wire		locked_bd ;
wire	[15:0] 	low16 ;
wire	[15:0] 	dia ;
wire	[15:0] 	dib ;
wire	[15:0] 	dic ;
wire	[15:0] 	did ;
wire	[1:0] 	dipa ;
wire	[1:0] 	dipb ;
wire	[1:0] 	dipc ;
wire	[1:0] 	dipd ;
reg 	[9:0] 	addra ;
reg 	[9:0] 	addrb ;
reg 	[9:0] 	addrc ;
reg 	[9:0] 	addrd ;
reg 		we_a ;
reg 		we_b ;
reg 		we_c ;
reg 		we_d ;
reg 		en_a ;
reg 		en_b ;
reg 		en_c ;
reg 		en_d ;
reg 		ssra ;
reg 		ssrb ;
reg 		ssrc ;
reg 		ssrd ;
wire	[15:0] 	dob ;
wire	[15:0] 	dod ;
wire	[1:0] 	dopb ;
wire	[1:0] 	dopd ;
wire		rst ;
wire		rstdcm ;
reg 	[17:0] 	countera ;
reg 	[17:0] 	counterc ;
reg 	[17:0] 	datab ;
reg 	[17:0] 	datad ;
reg 	[17:0] 	oldb ;
reg 	[17:0] 	oldd ;
reg 	[3:0] 	counta ;
reg 	[3:0] 	countc ;
wire		emptyab ;
wire		emptycd ;

assign low = 1'b0 ;
assign low16 = 16'h0000 ;
assign high = 1'b1 ;
assign locked = locked_ac & locked_bd ;
assign rst = ~rstin | ~(locked_ac & locked_bd) ;
assign rstdcm = ~rstin ;

IBUFG ibufga (.I(clkacin), .O(clkacint)) ;

DCM dcmac (
        .CLKIN(clkacint),
	.CLKFB(clkacx2),
	.RST(rstdcm),
	.CLK0(clkacdcm),
	.CLK2X(clkacx2dcm),
	.LOCKED(locked_ac));

BUFG bufgac (.I(clkacdcm), .O(clkac)) ;
BUFG bufgacx2 (.I(clkacx2dcm), .O(clkacx2)) ;

IBUFG ibufgb (.I(clkbdin), .O(clkbdint)) ;

DCM dcmbd (
        .CLKIN(clkbdint),
	.CLKFB(clkbdx2),
	.RST(rstdcm),
	.CLK0(clkbddcm),
	.CLK2X(clkbdx2dcm),
	.LOCKED(locked_bd));

BUFG bufgbd (.I(clkbddcm), .O(clkbd)) ;
BUFG bufgbdx2 (.I(clkbdx2dcm), .O(clkbdx2)) ;

ramb16_s18_s18_s18_s18 ram0 (
	.dia(dia),
	.dipa(dipa),
	.dib(dib),
	.dipb(dipb),
	.dic(dic),
	.dipc(dipc),
	.did(did),
	.dipd(dipd),
	.en_a(en_a),
	.en_b(en_b),
	.en_c(en_c),
	.en_d(en_d),
	.wea(we_a),
	.web(we_b),
	.wec(we_c),
	.wed(we_d),
	.ssra(ssra),
	.ssrb(ssrb),
	.ssrc(ssrc),
	.ssrd(ssrd),
	.clkac(clkac),
	.clkbd(clkbd),
	.clkacx2(clkacx2),
	.clkbdx2(clkbdx2),
	.rst(rst),
	.addra (addra),
	.addrb (addrb),
	.addrc (addrc),
	.addrd (addrd),
	.dob(dob),
	.dopb(dopb),
	.dod(dod),
	.dopd(dopd)) ;

assign dia = countera[15:0] ;
assign dipa = countera[17:16] ;

always @(posedge clkac or posedge rst)				// set up a test to write patterns to port A
begin
if (rst) begin
	countera <= 18'b000000000000000000 ;
	counta <= 4'b0000 ;
	addra <= 10'b0000000000 ;
	we_a <= 1'b0 ;
	en_a <= 1'b0 ;
	ssra <= 1'b1 ;
end
else begin
	counta <= counta + 4'b0001 ;
	en_a <= 1'b1 ;
	ssra <= 1'b0 ;
	if (counta < 4'b1110) begin
		countera <= countera + 18'b000000000000000001 ;
		addra[8:0] <= addra[8:0] + 9'b000000001 ;
		addra[9] <= 1'b0 ;
		we_a <= 1'b1 ;
	end else begin
		we_a <= 1'b0 ;
	end
end
end

assign dic = counterc[15:0] ;
assign dipc = counterc[17:16] ;

always @(posedge clkac or posedge rst)				// set up a test to write patterns to port C
begin
if (rst) begin
	counterc <= 18'b000000000000000000 ;
	countc <= 4'b0000 ;
	addrc <= 10'b1000000000 ;
	we_c <= 1'b0 ;
	en_c <= 1'b0 ;
	ssrc <= 1'b1 ;
end
else begin
	countc <= countc + 4'b0001 ;
	en_c <= 1'b1 ;
	ssrc <= 1'b0 ;
	if (countc < 4'b1101) begin
		counterc <= counterc - 18'b000000000000000001 ;
		addrc[8:0] <= addrc[8:0] + 9'b000000001 ;
		addrc[9] <= 1'b1 ;
		we_c <= 1'b1 ;
	end else begin
		we_c <= 1'b0 ;
	end
end
end

assign dib = 16'h0000 ;
assign dipb = 2'b00 ;

assign emptyab =(((addrb[8:7] == 2'b00) && (addra[8:7] == 2'b01)) |
		 ((addrb[8:7] == 2'b01) && (addra[8:7] == 2'b10)) |
		 ((addrb[8:7] == 2'b10) && (addra[8:7] == 2'b11)) |
		 ((addrb[8:7] == 2'b11) && (addra[8:7] == 2'b00)))
		 ? 1'b1 : 1'b0;

always @(posedge clkbd or posedge rst)				// read and check port B
begin
if (rst) begin
	datab <= 18'b000000000000000000 ;
	oldb <= 18'b000000000000000000 ;
	addrb <= 10'b0000000001 ;
	matchab <= 1'b0 ;
	we_b <= 1'b0 ;
	en_b <= 1'b0 ;
	ssrb <= 1'b1 ;
end
else begin
	ssrb <= 1'b0 ;
	if (en_b) begin
		datab <= {dopb, dob} ;
		oldb <= datab ;
	end
	if (emptyab) begin
		en_b <= 1'b1 ;
		addrb[8:0] <= addrb[8:0] + 9'b000000001 ;
		addrb[9] <= 1'b0 ;
	end else begin
		en_b <= 1'b0 ;
	end
	if (datab == (oldb + 18'b000000000000000001))
		matchab <= 1'b1 ;
	else
		matchab <= 1'b0 ;
end
end

assign did = 16'h0000 ;
assign dipd = 2'b00 ;

assign emptycd =(((addrd[8:7] == 2'b00) && (addrc[8:7] == 2'b01)) |
		 ((addrd[8:7] == 2'b01) && (addrc[8:7] == 2'b10)) |
		 ((addrd[8:7] == 2'b10) && (addrc[8:7] == 2'b11)) |
		 ((addrd[8:7] == 2'b11) && (addrc[8:7] == 2'b00)))
		 ? 1'b1 : 1'b0;

always @(posedge clkbd or posedge rst)				// read and check port D
begin
if (rst) begin
	datad <= 18'b000000000000000000 ;
	oldd <= 18'b000000000000000000 ;
	addrd <= 10'b1000000001 ;
	matchcd <= 1'b0 ;
	we_d <= 1'b0 ;
	en_d <= 1'b0 ;
	ssrd <= 1'b1 ;
end
else begin
	ssrd <= 1'b0 ;
	if (en_d) begin
		datad <= {dopd, dod} ;
		oldd <= datad ;
	end
	if (emptycd) begin
		en_d <= 1'b1 ;
		addrd[8:0] <= addrd[8:0] + 9'b000000001 ;
		addrd[9] <= 1'b1 ;
	end else begin
		en_d <= 1'b0 ;
	end
	if (datad == (oldd - 18'b000000000000000001))
		matchcd <= 1'b1 ;
	else
		matchcd <= 1'b0 ;
end
end
endmodule


