------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 228 - Quad port RAM in Virtex devices
--
------------------------------------------------------------------------------
--
--  File name :       top_v2.vhd
--
--  Description :     Example top-level for quad-port memory in VirtexII
--
--  Date - revision : 09/17/2002 - v 1.0
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors makeand you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specificallydisclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does notwarrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designswill be
--              uninterrupted or error free, or that defects in theDesigns
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results ofthe
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or forany
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on anytheory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure ofthe
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity top_v2 is port (
	rstin		: in std_logic ;	-- reset input (active low)
	clkacin 	: in std_logic;		-- clock for ports a and c
	clkbdin 	: in std_logic;		-- clock for ports b and d (faster in this example)
	matchab		: out std_logic ;	-- A to B transfer working
	matchcd		: out std_logic ;	-- C to D transfer working
	locked		: out std_logic);	-- DCMs locked
end top_v2;

architecture arch_top_v2 of top_v2 is
-- components
-- four port ram, all 512 x 16
component ramb16_s18_s18_s18_s18 port (
	DIA    	 	: in std_logic_vector (15 downto 0);	-- A data in
	DIB    	 	: in std_logic_vector (15 downto 0);	-- B data in
	DIC    	 	: in std_logic_vector (15 downto 0);	-- C data in
	DID    		: in std_logic_vector (15 downto 0);	-- D data in
	DIPA    	: in std_logic_vector (1 downto 0);	-- A parity in
	DIPB    	: in std_logic_vector (1 downto 0);	-- B parity in
	DIPC    	: in std_logic_vector (1 downto 0);	-- C parity in
	DIPD    	: in std_logic_vector (1 downto 0);	-- D parity in
	EN_A   		: in std_logic;				-- A EN in
	EN_B   		: in std_logic;				-- B EN in
	EN_C   		: in std_logic;				-- C EN in
	EN_D   		: in std_logic;				-- D EN in
	WEA    		: in std_logic;				-- A write enable in
	WEB    		: in std_logic;				-- B write enable in
	WEC    		: in std_logic;				-- C write enable in
	WED    		: in std_logic;				-- D write enable in
	SSRA   		: in std_logic;				-- A sync set/reset in
	SSRB   		: in std_logic;				-- B sync set/reset in
	SSRC   		: in std_logic;				-- C sync set/reset in
	SSRD   		: in std_logic;				-- D sync set/reset in
	clkac 		: in std_logic;				-- port A and C clock
	clkbd 		: in std_logic;				-- port B and D clock
	clkacx2 	: in std_logic;				-- port A and C clock x2
	clkbdx2 	: in std_logic;				-- port B and D clock x2
	rst		: in std_logic;				-- reset
	ADDRA  		: in std_logic_vector (9 downto 0);	-- A address in
	ADDRB  		: in std_logic_vector (9 downto 0);	-- B address in
	ADDRC  		: in std_logic_vector (9 downto 0);	-- C address in
	ADDRD  		: in std_logic_vector (9 downto 0);	-- D address in
	DOA    		: out std_logic_vector (15 downto 0);	-- A data out
	DOB    		: out std_logic_vector (15 downto 0);	-- B data out
	DOC    		: out std_logic_vector (15 downto 0);	-- C data out
	DOD    		: out std_logic_vector (15 downto 0);	-- D data out
	DOPA    	: out std_logic_vector (1 downto 0);	-- A parity out
	DOPB    	: out std_logic_vector (1 downto 0);	-- B parity out
	DOPC    	: out std_logic_vector (1 downto 0);	-- C parity out
	DOPD    	: out std_logic_vector (1 downto 0));	-- D parity out
end component ;
-- DCM primitive
component dcm port (
	clkin   	: in  std_logic ;
	CLKFB   	: in  std_logic ;
	DSSEN 		: in  std_logic ;
	PSINCDEC	: in  std_logic ;
	PSEN 		: in  std_logic ;
	PSCLK 		: in  std_logic ;
	RST     	: in  std_logic ;
	CLK0    	: out std_logic ;
	CLK90   	: out std_logic ;
	CLK180  	: out std_logic ;
	CLK270  	: out std_logic ;
	CLK2X   	: out std_logic ;
	CLK2X180	: out std_logic ;
	CLKDV   	: out std_logic ;
	CLKFX   	: out std_logic ;
	CLKFX180	: out std_logic ;
	LOCKED  	: out std_logic ;
	PSDONE  	: out std_logic ;
	STATUS  	: out std_logic_vector(7 downto 0));
end component ;
-- BUFG primitive
component bufg port(i: in std_logic; o: out std_logic); end component;
-- IBUFG primitive
component ibufg port(i: in std_logic; o: out std_logic); end component;

signal high 		: std_logic ;
signal low 		: std_logic ;
signal clkacint 	: std_logic ;
signal clkbdint 	: std_logic ;
signal clkacx2 		: std_logic ;
signal clkbdx2 		: std_logic ;
signal clkacdcm 	: std_logic ;
signal clkbddcm 	: std_logic ;
signal clkacx2dcm 	: std_logic ;
signal clkbdx2dcm 	: std_logic ;
signal clkac 		: std_logic ;
signal clkbd 		: std_logic ;
signal locked_ac 	: std_logic ;
signal locked_bd 	: std_logic ;
signal low16 		: std_logic_vector(15 downto 0);
signal low2 		: std_logic_vector(1 downto 0);
signal dia 		: std_logic_vector (15 downto 0) ;
signal dib 		: std_logic_vector (15 downto 0) ;
signal dic 		: std_logic_vector (15 downto 0) ;
signal did 		: std_logic_vector (15 downto 0) ;
signal dipa 		: std_logic_vector (1 downto 0) ;
signal dipb 		: std_logic_vector (1 downto 0) ;
signal dipc 		: std_logic_vector (1 downto 0) ;
signal dipd 		: std_logic_vector (1 downto 0) ;
signal addra 		: std_logic_vector (9 downto 0) ;
signal addrb 		: std_logic_vector (9 downto 0) ;
signal addrc 		: std_logic_vector (9 downto 0) ;
signal addrd 		: std_logic_vector (9 downto 0) ;
signal we_a 		: std_logic ;
signal we_b 		: std_logic ;
signal we_c 		: std_logic ;
signal we_d 		: std_logic ;
signal en_a 		: std_logic ;
signal en_b 		: std_logic ;
signal en_c 		: std_logic ;
signal en_d 		: std_logic ;
signal ssra 		: std_logic ;
signal ssrb 		: std_logic ;
signal ssrc 		: std_logic ;
signal ssrd 		: std_logic ;
signal dob 		: std_logic_vector (15 downto 0) ;
signal dopb 		: std_logic_vector (1 downto 0) ;
signal dod 		: std_logic_vector (15 downto 0) ;
signal dopd 		: std_logic_vector (1 downto 0) ;
signal rst 		: std_logic ;
signal rstdcm 		: std_logic ;
signal countera 	: std_logic_vector (17 downto 0) ;
signal counterc		: std_logic_vector (17 downto 0) ;
signal datab		: std_logic_vector (17 downto 0) ;
signal datad		: std_logic_vector (17 downto 0) ;
signal oldb		: std_logic_vector (17 downto 0) ;
signal oldd		: std_logic_vector (17 downto 0) ;
signal counta		: std_logic_vector (3 downto 0) ;
signal countc		: std_logic_vector (3 downto 0) ;
signal emptyab 		: std_logic ;
signal emptycd 		: std_logic ;

begin

low <= '0' ;
low2 <= "00" ;
low16 <= "0000000000000000" ;
high <= '1' ;
locked <= locked_ac and locked_bd ;
rst <= not rstin or not (locked_ac and locked_bd) ;
rstdcm <= not rstin ;

ibufga : ibufg port map (i => clkacin, o => clkacint) ;

dcma: dcm port map (
        clkin   	=> clkacint,
	CLKFB   	=> clkacx2,
	DSSEN 		=> low,
	PSINCDEC	=> low,
	PSEN 		=> low,
	PSCLK 		=> low,
	RST     	=> rstdcm,
	CLK0    	=> clkacdcm,
	CLK90   	=> open,
	CLK180  	=> open,
	CLK270  	=> open,
	CLK2X   	=> clkacx2dcm,
	CLK2X180	=> open,
	CLKDV   	=> open,
	CLKFX   	=> open,
	CLKFX180	=> open,
	LOCKED  	=> locked_ac,
	PSDONE  	=> open,
	STATUS  	=> open);

bufga : bufg port map (i => clkacdcm, o	=> clkac) ;
bufgax2 : bufg port map (i => clkacx2dcm, o => clkacx2) ;

ibufgb : ibufg port map (i => clkbdin, o => clkbdint) ;

dcmb: dcm port map (
        clkin   	=> clkbdint,
	CLKFB   	=> clkbdx2,
	DSSEN 		=> low,
	PSINCDEC	=> low,
	PSEN 		=> low,
	PSCLK 		=> low,
	RST     	=> rstdcm,
	CLK0    	=> clkbddcm,
	CLK90   	=> open,
	CLK180  	=> open,
	CLK270  	=> open,
	CLK2X   	=> clkbdx2dcm,
	CLK2X180	=> open,
	CLKDV   	=> open,
	CLKFX   	=> open,
	CLKFX180	=> open,
	LOCKED  	=> locked_bd,
	PSDONE  	=> open,
	STATUS  	=> open);

bufgb : bufg port map (i => clkbddcm, o	=> clkbd) ;
bufgbx2 : bufg port map (i => clkbdx2dcm, o => clkbdx2) ;

ram0 : ramb16_s18_s18_s18_s18 port map (
	DIA    		=> dia,
	DIB    		=> dib,
	DIC    		=> dic,
	DID    		=> did,
	DIPA    	=> dipa,
	DIPB    	=> dipb,
	DIPC    	=> dipc,
	DIPD    	=> dipd,
	EN_A   		=> en_a,
	EN_B   		=> en_b,
	EN_C   		=> en_c,
	EN_D   		=> en_d,
	WEA    		=> we_a,
	WEB    		=> we_b,
	WEC    		=> we_c,
	WED    		=> we_d,
	SSRA   		=> ssra,
	SSRB   		=> ssrb,
	SSRC   		=> ssrc,
	SSRD   		=> ssrd,
	clkac 		=> clkac,
	clkbd 		=> clkbd,
	clkacx2 	=> clkacx2,
	clkbdx2 	=> clkbdx2,
	rst		=> rst,
	ADDRA  		=> addra,
	ADDRB  		=> addrb,
	ADDRC  		=> addrc,
	ADDRD  		=> addrd,
	DOA    		=> open,
	DOB    		=> dob,
	DOC    		=> open,
	DOD    		=> dod,
	DOPA    	=> open,
	DOPB    	=> dopb,
	DOPC    	=> open,
	DOPD    	=> dopd);

dia <= countera(15 downto 0) ;
dipa <= countera(17 downto 16) ;

process (clkac, rst)				-- set up a test to write patterns to port A
begin
if rst = '1' then
	countera <= "000000000000000000" ;
	counta <= "0000" ;
	addra <= "0000000000" ;
	we_a <= '0' ;
	en_a <= '0' ;
	ssra <= '1' ;
elsif clkac'event and clkac = '1' then
	counta <= counta + 1 ;
	en_a <= '1' ;
	ssra <= '0' ;
	if counta < "1101" then
		countera <= countera + 1 ;
		addra(8 downto 0) <= addra(8 downto 0) + 1 ;
		we_a <= '1' ;
	else
		we_a <= '0' ;
	end if ;
end if ;
end process ;

dic <= counterc(15 downto 0) ;
dipc <= counterc(17 downto 16) ;

process (clkac, rst)				-- set up a test to write patterns to port C
begin
if rst = '1' then
	counterc <= "000000000000000000" ;
	countc <= "0000" ;
	addrc <= "1000000000" ;
	we_c <= '0' ;
	en_c <= '0' ;
	ssrc <= '1' ;
	countc <= "0000" ;
elsif clkac'event and clkac = '1' then
	countc <= countc + 1 ;
	en_c <= '1' ;
	ssrc <= '0' ;
	if countc < "1110" then
	 	counterc <= counterc - 1 ;
		addrc(8 downto 0) <= addrc(8 downto 0) + 1 ;
		we_c <= '1' ;
	else
		we_c <= '0' ;
	end if ;
end if ;
end process ;

dib <= "0000000000000000" ;
dipb <= "00" ;

emptyab <= '1' when addrb(8 downto 7) = "00" and addra(8 downto 7) = "01" else
           '1' when addrb(8 downto 7) = "01" and addra(8 downto 7) = "10" else
           '1' when addrb(8 downto 7) = "10" and addra(8 downto 7) = "11" else
           '1' when addrb(8 downto 7) = "11" and addra(8 downto 7) = "00" else
           '0' ;

process (clkbd, rst)				-- read and check port B
begin
if rst = '1' then
	datab <= "000000000000000000" ;
	oldb <= "000000000000000000" ;
	addrb <= "0000000000" ;
	matchab <= '0' ;
	en_b <= '0' ;
	ssrb <= '1' ;
	we_b <= '0' ;
elsif clkbd'event and clkbd = '1' then
	ssrb <= '0' ;
	if en_b = '1' then
		datab <= dopb & dob ;
		oldb <= datab ;
	end if ;
	if emptyab = '1' then
		en_b <= '1' ;
		addrb(8 downto 0) <= addrb(8 downto 0) + 1 ;
	else
		en_b <= '0' ;
	end if ;
	if datab = oldb + 1 then
		matchab <= '1' ;
	else
		matchab <= '0' ;
	end if ;
end if ;
end process ;

did <= "0000000000000000" ;
dipd <= "00" ;

emptycd <= '1' when addrd(8 downto 7) = "00" and addrc(8 downto 7) = "01" else
           '1' when addrd(8 downto 7) = "01" and addrc(8 downto 7) = "10" else
           '1' when addrd(8 downto 7) = "10" and addrc(8 downto 7) = "11" else
           '1' when addrd(8 downto 7) = "11" and addrc(8 downto 7) = "00" else
           '0' ;

process (clkbd, rst)				-- read and check port D
begin
if rst = '1' then
	datad <= "000000000000000000" ;
	oldd <= "000000000000000000" ;
	addrd <= "1000000000" ;
	matchcd <= '0' ;
	en_d <= '0' ;
	ssrd <= '1' ;
	we_d <= '0' ;
elsif clkbd'event and clkbd = '1' then
	ssrd <= '0' ;
	if en_d = '1' then
		datad <= dopd & dod ;
		oldd <= datad ;
	end if ;
	if emptycd = '1' then
		en_d <= '1' ;
		addrd(8 downto 0) <= addrd(8 downto 0) + 1 ;
	else
		en_d <= '0' ;
	end if ;
	if datad = oldd - 1 then
		matchcd <= '1' ;
	else
		matchcd <= '0' ;
	end if ;
end if ;
end process ;
end arch_top_v2;


