//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2002                 www.xilinx.com
//
//  XAPP 228 - Quad port RAM in Virtex devices
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :       ramb4_s16_s16_s16_s16.v
//
//  Description :     Verilog for a four port 256 x 16 memory in VirtexE
//
//  Date - revision : June 4th 2003 - v 1.1
//
//  Author :          NJS
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors makeand you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specificallydisclaims any
//              implied warranties of merchantability, non-infringement,or
//              fitness for a particular purpose. Xilinx does notwarrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designswill be
//              uninterrupted or error free, or that defects in theDesigns
//              will be corrected. Furthermore, Xilinx does not warrantor
//              make any representations regarding use or the results ofthe
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits,cost
//              or procurement of substitute goods or services, or forany
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on anytheory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstanding the failure ofthe
//              essential purpose of any limited remedies herein.
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// This module emulates a four port memory using 2x clocks
// two full read/write ports (A and C) in clock A domain
// two full read/write ports (B and D) in clock B domain
//
`timescale 1 ps / 1ps

module ramb4_s16_s16_s16_s16 (
	dia,			// A data in
	dib,			// B data in
	dic,			// C data in
	did,			// D data in
	en_a,			// A EN in
	en_b,			// B EN in
	en_c,			// C EN in
	en_d,			// D EN in
	wea,			// A write enable in
	web,			// B write enable in
	wec,			// C write enable in
	wed,			// D write enable in
	rsta,			// A reset in
	rstb,			// B reset in
	rstc,			// C reset in
	rstd,			// D reset in
	clkac,			// port A and C clock
	clkbd,			// port B and D clock
	clkacx2,		// port A and C clock x2
	clkbdx2,		// port B and D clock x2
	rst,			// reset
	addra,			// A address in
	addrb,			// B address in
	addrc,			// C address in
	addrd,			// D address in
	doa,			// A data out
	dob,			// B data out
	doc,			// C data out
	dod) ;			// D data out

input 		en_a, en_b, en_c, en_d, wea, web, wec, wed, rsta, rstb, rstc, rstd, clkac, clkbd ;
input 		clkacx2, clkbdx2, rst ;
input 	[15:0] 	dia, dib, dic, did ;
input 	[7:0] 	addra, addrb, addrc, addrd ;

output 	[15:0] 	doa, dob, doc, dod ;

reg 	[15:0] 	doa, dob, doc, dod ;

wire 		high  ;
wire 		low  ;
wire 	[15:0] 	data_muxac ;
wire 	[15:0] 	data_muxbd ;
wire 	[15:0] 	doa_int ;
wire 	[15:0] 	dob_int ;
wire 		en_muxac ;
wire 		en_muxbd ;
wire 		we_muxac ;
wire 		we_muxbd ;
wire 		rst_muxac ;
wire 		rst_muxbd ;
wire 	[7:0] 	addr_muxac ;
wire 	[7:0] 	addr_muxbd ;
wire 		muxac ;
wire 		muxbd ;
wire 		clkac_lac ;
wire 		clkbd_lac ;
wire 		notclkac ;
wire 		notclkbd ;
reg 		en_c_del ;
reg 		en_d_del ;

assign low = 1'b0 ;
assign high = 1'b1 ;
assign notclkac = ~clkac ;
assign notclkbd = ~clkbd ;

// Generate logic available clocks
FDC clkac_lacff (.C(clkacx2), .D(notclkac), .CLR(rst), .Q(clkac_lac));
FDC clkbd_lacff (.C(clkbdx2), .D(notclkbd), .CLR(rst), .Q(clkbd_lac));

RAMB4_S16_S16 block_ram  (
	.DIA(data_muxac) ,
	.DIB(data_muxbd),
	.ENA(en_muxac),
	.ENB(en_muxbd),
	.WEA(we_muxac),
	.WEB(we_muxbd),
	.RSTA(rst_muxac),
	.RSTB(rst_muxbd),
	.CLKA(clkacx2),
	.CLKB(clkbdx2),
	.ADDRA(addr_muxac),
	.ADDRB(addr_muxbd),
	.DOA(doa_int),
	.DOB(dob_int)) ;

assign data_muxac =(muxac) ? dic : dia ;
assign data_muxbd =(muxbd) ? did : dib ;

assign we_muxac =(muxac) ? wec : wea ;
assign we_muxbd =(muxbd) ? wed : web ;

assign en_muxac =(muxac) ? en_c : en_a ;
assign en_muxbd =(muxbd) ? en_d : en_b ;

assign rst_muxac =(muxac) ? rstc : rsta ;
assign rst_muxbd =(muxbd) ? rstd : rstb ;

assign addr_muxac =(muxac) ? addrc : addra ;
assign addr_muxbd =(muxbd) ? addrd : addrb ;

assign muxac = clkac_lac ;

always @(posedge clkac or posedge rst)
begin
if (rst) begin
	doa <= 16'h0000 ;
end else begin
	if (en_a) begin
		doa <= doa_int ;
	end
end
end

always @(negedge clkac or posedge rst)
begin
if (rst) begin
	doc <= 16'h0000 ;
	en_c_del <= 1'b0 ;
end else begin
	en_c_del <= en_c ;
	if (en_c_del) begin
		doc <= doa_int ;
	end
end
end

assign muxbd = clkbd_lac ;

always @(posedge clkbd or posedge rst)
begin
if (rst) begin
	dob <= 16'h0000 ;
end else begin
	if (en_b) begin
		dob <= dob_int ;
	end
end
end

always @(negedge clkbd or posedge rst)
begin
if (rst) begin
	dod <= 16'h0000 ;
	en_d_del <= 1'b0 ;
end else begin
	en_d_del <= en_d ;
	if (en_d_del) begin
		dod <= dob_int ;
	end
end
end
endmodule


