------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 228 - Quad port RAM in Virtex devices
--
------------------------------------------------------------------------------
--
--  File name :       ramb4_s16_s16_s16_s16.vhd
--
--  Description :     VHDL for a four port 256 x 16 memory in VirtexE
--
--  Date - revision : 09/17/2002 - v 1.0
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors makeand you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specificallydisclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does notwarrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designswill be
--              uninterrupted or error free, or that defects in theDesigns
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results ofthe
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or forany
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on anytheory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure ofthe
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
-------------------------------------------------------------------------------
-- This module emulates a four port memory using 2x clocks
-- two full read/write ports (A and C) in clock A domain
-- two full read/write ports (B and D) in clock B domain
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity ramb4_s16_s16_s16_s16 is port (
	DIA    	 	: in std_logic_vector (15 downto 0);	-- A data in
	DIB    	 	: in std_logic_vector (15 downto 0);	-- B data in
	DIC    	 	: in std_logic_vector (15 downto 0);	-- C data in
	DID    		: in std_logic_vector (15 downto 0);	-- D data in
	EN_A   		: in std_logic;				-- A EN in
	EN_B   		: in std_logic;				-- B EN in
	EN_C   		: in std_logic;				-- C EN in
	EN_D   		: in std_logic;				-- D EN in
	WEA    		: in std_logic;				-- A write enable in
	WEB    		: in std_logic;				-- B write enable in
	WEC    		: in std_logic;				-- C write enable in
	WED    		: in std_logic;				-- D write enable in
	RSTA   		: in std_logic;				-- A reset in
	RSTB   		: in std_logic;				-- B reset in
	RSTC   		: in std_logic;				-- C reset in
	RSTD   		: in std_logic;				-- D reset in
	clkac 		: in std_logic;				-- port A and C clock
	clkbd 		: in std_logic;				-- port B and D clock
	clkacx2 	: in std_logic;				-- port A and C clock x2
	clkbdx2 	: in std_logic;				-- port B and D clock x2
	rst		: in std_logic;				-- reset
	ADDRA  		: in std_logic_vector (7 downto 0);	-- A address in
	ADDRB  		: in std_logic_vector (7 downto 0);	-- B address in
	ADDRC  		: in std_logic_vector (7 downto 0);	-- C address in
	ADDRD  		: in std_logic_vector (7 downto 0);	-- D address in
	DOA    		: out std_logic_vector (15 downto 0);	-- A data out
	DOB    		: out std_logic_vector (15 downto 0);	-- B data out
	DOC    		: out std_logic_vector (15 downto 0);	-- C data out
	DOD    		: out std_logic_vector (15 downto 0));	-- D data out
end ramb4_s16_s16_s16_s16;

architecture arch_ramb4_s16_s16_s16_s16 of ramb4_s16_s16_s16_s16 is
-- ramb4_s16_s16 primitive
component ramb4_s16_s16 port (
	DIA    		: in std_logic_vector (15 downto 0);
	DIB    		: in std_logic_vector (15 downto 0);
	ENA    		: in std_logic;
	ENB    		: in std_logic;
	WEA    		: in std_logic;
	WEB    		: in std_logic;
	rstA   		: in std_logic;
	rstB   		: in std_logic;
	clkA   		: in std_logic;
	clkB   		: in std_logic;
	ADDRA  		: in std_logic_vector (7 downto 0);
	ADDRB  		: in std_logic_vector (7 downto 0);
	DOA    		: out std_logic_vector (15 downto 0);
	DOB    		: out std_logic_vector (15 downto 0));
end component ;
-- FDC primitive
component fdc port(c, d, clr : in std_logic; q: out std_logic); end component;

signal high 		: std_logic ;
signal low 		: std_logic ;
signal data_muxac 	: std_logic_vector (15 downto 0);
signal data_muxbd 	: std_logic_vector (15 downto 0);
signal doa_int 		: std_logic_vector (15 downto 0);
signal dob_int 		: std_logic_vector (15 downto 0);
signal en_muxac 	: std_logic;
signal en_muxbd 	: std_logic;
signal we_muxac 	: std_logic;
signal we_muxbd 	: std_logic;
signal rst_muxac 	: std_logic;
signal rst_muxbd 	: std_logic;
signal addr_muxac 	: std_logic_vector (7 downto 0);
signal addr_muxbd 	: std_logic_vector (7 downto 0);
signal muxac 		: std_logic;
signal muxbd 		: std_logic;
signal en_c_del		: std_logic;
signal en_d_del		: std_logic;
signal clkac_lac	: std_logic;
signal clkbd_lac	: std_logic;
signal notclkac		: std_logic;
signal notclkbd		: std_logic;

begin

low <= '0' ;
high <= '1' ;
notclkac <= not clkac ;
notclkbd <= not clkbd ;

-- Generate logic available clocks
clkac_lacff : fdc port map (c => clkacx2, d => notclkac, clr => rst, q => clkac_lac);
clkbd_lacff : fdc port map (c => clkbdx2, d => notclkbd, clr => rst, q => clkbd_lac);

block_ram : ramb4_s16_s16 port map (
	DIA    		=> data_muxac ,
	DIB    		=> data_muxbd,
	ENA    		=> en_muxac,
	ENB    		=> en_muxbd,
	WEA    		=> we_muxac,
	WEB    		=> we_muxbd,
	rstA   		=> rst_muxac,
	rstB   		=> rst_muxbd,
	clkA   		=> clkacx2,
	clkB   		=> clkbdx2,
	ADDRA  		=> addr_muxac,
	ADDRB  		=> addr_muxbd,
	DOA    		=> doa_int,
	DOB    		=> dob_int) ;

data_muxac <= dic when muxac = '0' else dia ;
data_muxbd <= did when muxbd = '0' else dib ;

we_muxac <= wec when muxac = '0' else wea ;
we_muxbd <= wed when muxbd = '0' else web ;

en_muxac <= en_c when muxac = '0' else en_a ;
en_muxbd <= en_d when muxbd = '0' else en_b ;

rst_muxac <= rstc when muxac = '0' else rsta ;
rst_muxbd <= rstd when muxbd = '0' else rstb ;

addr_muxac <= addrc when muxac = '0' else addra ;
addr_muxbd <= addrd when muxbd = '0' else addrb ;

muxac <= clkac_lac ;

process (clkac, rst)
begin
if rst = '1' then
	doa <= (others => '0') ;
elsif clkac'event and clkac = '1' then
	if en_a = '1' then
		doa <= doa_int ;
	end if ;
end if ;
end process ;

process (clkac, rst)
begin
if rst = '1' then
	doc <= (others => '0') ;
	en_c_del <= '0' ;
elsif clkac'event and clkac = '0' then
	en_c_del <= en_c ;
	if en_c_del = '1' then
		doc <= doa_int ;
	end if ;
end if ;
end process ;

muxbd <= clkbd_lac ;

process (clkbd, rst)
begin
if rst = '1' then
	dob <= (others => '0') ;
elsif clkbd'event and clkbd = '1' then
	if en_b = '1' then
		dob <= dob_int ;
	end if ;
end if ;
end process ;

process (clkbd, rst)
begin
if rst = '1' then
	dod <= (others => '0') ;
	en_d_del <= '0' ;
elsif clkbd'event and clkbd = '0' then
	en_d_del <= en_d ;
	if en_d_del = '1' then
		dod <= dob_int ;
	end if ;
end if ;
end process ;

end arch_ramb4_s16_s16_s16_s16;


