------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 228 - Quad port RAM in Virtex devices
--
------------------------------------------------------------------------------
--
--  File name :       top_ve.vhd
--
--  Description :     Example top-level for quad-port memory in VirtexE
--
--  Date - revision : 09/17/2002 - v 1.0
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors makeand you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specificallydisclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does notwarrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designswill be
--              uninterrupted or error free, or that defects in theDesigns
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results ofthe
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or forany
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on anytheory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure ofthe
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity top_ve is port (
	rstin		: in std_logic ;	-- reset input (active low)
	clkacin 	: in std_logic;		-- clock for ports a and c
	clkbdin 	: in std_logic;		-- clock for ports b and d (faster in this example)
	matchab		: out std_logic ;	-- A to B transfer working
	matchcd		: out std_logic ;	-- C to D transfer working
	locked		: out std_logic);	-- dlls locked
end top_ve;

architecture arch_top_ve of top_ve is
-- components
-- four port ram
component ramb4_s16_s16_s16_s16 port (
	DIA    	 	: in std_logic_vector (15 downto 0);	-- A data in
	DIB    	 	: in std_logic_vector (15 downto 0);	-- B data in
	DIC    	 	: in std_logic_vector (15 downto 0);	-- C data in
	DID    		: in std_logic_vector (15 downto 0);	-- D data in
	EN_A   		: in std_logic;				-- A EN in
	EN_B   		: in std_logic;				-- B EN in
	EN_C   		: in std_logic;				-- C EN in
	EN_D   		: in std_logic;				-- D EN in
	WEA    		: in std_logic;				-- A write enable in
	WEB    		: in std_logic;				-- B write enable in
	WEC    		: in std_logic;				-- C write enable in
	WED    		: in std_logic;				-- D write enable in
	RSTA   		: in std_logic;				-- A reset in
	RSTB   		: in std_logic;				-- B reset in
	RSTC   		: in std_logic;				-- C reset in
	RSTD   		: in std_logic;				-- D reset in
	clkac 		: in std_logic;				-- port A and C clock
	clkbd 		: in std_logic;				-- port B and D clock
	clkacx2 	: in std_logic;				-- port A and C clock x2
	clkbdx2 	: in std_logic;				-- port B and D clock x2
	rst		: in std_logic;				-- reset
	ADDRA  		: in std_logic_vector (7 downto 0);	-- A address in
	ADDRB  		: in std_logic_vector (7 downto 0);	-- B address in
	ADDRC  		: in std_logic_vector (7 downto 0);	-- C address in
	ADDRD  		: in std_logic_vector (7 downto 0);	-- D address in
	DOA    		: out std_logic_vector (15 downto 0);	-- A data out
	DOB    		: out std_logic_vector (15 downto 0);	-- B data out
	DOC    		: out std_logic_vector (15 downto 0);	-- C data out
	DOD    		: out std_logic_vector (15 downto 0));	-- D data out
end component ;
-- Low frequency DLL primitive
component clkdll port (
	clkin 		: in std_logic;
	clkfb 		: in std_logic;
	rst 		: in std_logic;
	clk0 		: out std_logic;
	clk90 		: out std_logic;
	clk180 		: out std_logic;
	clk270 		: out std_logic;
	clk2x 		: out std_logic;
	clkdv 		: out std_logic;
	locked 		: out std_logic );
end component ;
-- BUFG primitive
component bufg port(i: in std_logic; o: out std_logic); end component;
-- IBUFG primitive
component ibufg port(i: in std_logic; o: out std_logic); end component;
-- FDC primitive
component fdc port(c, d, clr : in std_logic; q: out std_logic); end component;

signal high 		: std_logic ;
signal low 		: std_logic ;
signal clkacint 	: std_logic ;
signal clkbdint 	: std_logic ;
signal clkacx2 		: std_logic ;
signal clkbdx2 		: std_logic ;
signal clkacdll 	: std_logic ;
signal clkbddll 	: std_logic ;
signal clkacx2dll 	: std_logic ;
signal clkbdx2dll 	: std_logic ;
signal clkac 		: std_logic ;
signal clkbd 		: std_logic ;
signal locked_ac 	: std_logic ;
signal locked_bd 	: std_logic ;
signal low16 		: std_logic_vector(15 downto 0);
signal dia 		: std_logic_vector (15 downto 0) ;
signal dib 		: std_logic_vector (15 downto 0) ;
signal dic 		: std_logic_vector (15 downto 0) ;
signal did 		: std_logic_vector (15 downto 0) ;
signal dipa 		: std_logic_vector (1 downto 0) ;
signal dipb 		: std_logic_vector (1 downto 0) ;
signal dipc 		: std_logic_vector (1 downto 0) ;
signal dipd 		: std_logic_vector (1 downto 0) ;
signal addra 		: std_logic_vector (7 downto 0) ;
signal addrb 		: std_logic_vector (7 downto 0) ;
signal addrc 		: std_logic_vector (7 downto 0) ;
signal addrd 		: std_logic_vector (7 downto 0) ;
signal we_a 		: std_logic ;
signal we_b 		: std_logic ;
signal we_c 		: std_logic ;
signal we_d 		: std_logic ;
signal en_a 		: std_logic ;
signal en_b 		: std_logic ;
signal en_c 		: std_logic ;
signal en_d 		: std_logic ;
signal rsta 		: std_logic ;
signal rstb 		: std_logic ;
signal rstc 		: std_logic ;
signal rstd 		: std_logic ;
signal dob 		: std_logic_vector (15 downto 0) ;
signal dod 		: std_logic_vector (15 downto 0) ;
signal rst 		: std_logic ;
signal rstdll 		: std_logic ;
signal countera 	: std_logic_vector (15 downto 0) ;
signal counterc		: std_logic_vector (15 downto 0) ;
signal datab		: std_logic_vector (15 downto 0) ;
signal datad		: std_logic_vector (15 downto 0) ;
signal oldb		: std_logic_vector (15 downto 0) ;
signal oldd		: std_logic_vector (15 downto 0) ;
signal counta		: std_logic_vector (3 downto 0) ;
signal countc		: std_logic_vector (3 downto 0) ;
signal emptyab 		: std_logic ;
signal emptycd 		: std_logic ;

begin

low <= '0' ;
low16 <= "0000000000000000" ;
high <= '1' ;
locked <= locked_ac and locked_bd ;
rst <= not rstin or not (locked_ac and locked_bd) ;
rstdll <= not rstin ;

ibufga : ibufg port map (i => clkacin, o => clkacint) ;

dlla: clkdll port map (
        clkin   	=> clkacint,
	CLKFB   	=> clkac,
	RST     	=> rstdll,
	CLK0    	=> clkacdll,
	CLK90   	=> open,
	CLK180  	=> open,
	CLK270  	=> open,
	CLK2X  		=> clkacx2dll,
	CLKDV   	=> open,
	LOCKED  	=> locked_ac);

bufga : bufg port map (i => clkacdll, o	=> clkac) ;
bufgax2 : bufg port map (i => clkacx2dll, o => clkacx2) ;

ibufgb : ibufg port map (i => clkbdin, o => clkbdint) ;

dllb: clkdll port map (
        clkin   	=> clkbdint,
	CLKFB   	=> clkbd,
	RST     	=> rstdll,
	CLK0    	=> clkbddll,
	CLK90   	=> open,
	CLK180  	=> open,
	CLK270  	=> open,
	CLK2X   	=> clkbdx2dll,
	CLKDV   	=> open,
	LOCKED  	=> locked_bd);

bufgb : bufg port map (i => clkbddll, o	=> clkbd) ;
bufgbx2 : bufg port map (i => clkbdx2dll, o => clkbdx2) ;

ram0 : ramb4_s16_s16_s16_s16 port map (
	DIA    		=> dia,
	DIB    		=> dib,
	DIC    		=> dic,
	DID    		=> did,
	EN_A   		=> en_a,
	EN_B   		=> en_b,
	EN_C   		=> en_c,
	EN_D   		=> en_d,
	WEA    		=> we_a,
	WEB    		=> we_b,
	WEC    		=> we_c,
	WED    		=> we_d,
	RSTA   		=> rsta,
	RSTB   		=> rstb,
	RSTC   		=> rstc,
	RSTD   		=> rstd,
	clkac 		=> clkac,
	clkbd 		=> clkbd,
	clkacx2 	=> clkacx2,
	clkbdx2 	=> clkbdx2,
	rst		=> rst,
	ADDRA  		=> addra,
	ADDRB  		=> addrb,
	ADDRC  		=> addrc,
	ADDRD  		=> addrd,
	DOA    		=> open,
	DOB    		=> dob,
	DOC    		=> open,
	DOD    		=> dod) ;

dia <= countera ;

process (clkac, rst)				-- set up a test to write patterns to port A
begin
if rst = '1' then
	countera <= "0000000000000000" ;
	counta <= "0000" ;
	addra <= "00000000" ;
	we_a <= '0' ;
	en_a <= '0' ;
	rsta <= '1' ;
elsif clkac'event and clkac = '1' then
	counta <= counta + 1 ;
	en_a <= '1' ;
	rsta <= '0' ;
	if counta < "1101" then
		countera <= countera + 1 ;
		addra(6 downto 0) <= addra(6 downto 0) + 1 ;
		we_a <= '1' ;
	else
		we_a <= '0' ;
	end if ;
end if ;
end process ;

dic <= counterc ;

process (clkac, rst)				-- set up a test to write patterns to port C
begin
if rst = '1' then
	counterc <= "0000000000000000" ;
	countc <= "0000" ;
	addrc <= "10000000" ;
	we_c <= '0' ;
	en_c <= '0' ;
	rstc <= '1' ;
	countc <= "0000" ;
elsif clkac'event and clkac = '1' then
	countc <= countc + 1 ;
	en_c <= '1' ;
	rstc <= '0' ;
	if countc < "1110" then
	 	counterc <= counterc - 1 ;
		addrc(6 downto 0) <= addrc(6 downto 0) + 1 ;
		we_c <= '1' ;
	else
		we_c <= '0' ;
	end if ;
end if ;
end process ;

dib <= "0000000000000000" ;

emptyab <= '1' when addrb(6 downto 5) = "00" and addra(6 downto 5) = "01" else
           '1' when addrb(6 downto 5) = "01" and addra(6 downto 5) = "10" else
           '1' when addrb(6 downto 5) = "10" and addra(6 downto 5) = "11" else
           '1' when addrb(6 downto 5) = "11" and addra(6 downto 5) = "00" else
           '0' ;

process (clkbd, rst)				-- read and check port B
begin
if rst = '1' then
	datab <= "0000000000000000" ;
	oldb <= "0000000000000000" ;
	addrb <= "00000000" ;
	matchab <= '0' ;
	en_b <= '0' ;
	rstb <= '1' ;
	we_b <= '0' ;
elsif clkbd'event and clkbd = '1' then
	rstb <= '0' ;
	if en_b = '1' then
		datab <= dob ;
		oldb <= datab ;
	end if ;
	if emptyab = '1' then
		en_b <= '1' ;
		addrb(6 downto 0) <= addrb(6 downto 0) + 1 ;
	else
		en_b <= '0' ;
	end if ;
	if datab = oldb + 1 then
		matchab <= '1' ;
	else
		matchab <= '0' ;
	end if ;
end if ;
end process ;

did <= "0000000000000000" ;

emptycd <= '1' when addrd(6 downto 5) = "00" and addrc(6 downto 5) = "01" else
           '1' when addrd(6 downto 5) = "01" and addrc(6 downto 5) = "10" else
           '1' when addrd(6 downto 5) = "10" and addrc(6 downto 5) = "11" else
           '1' when addrd(6 downto 5) = "11" and addrc(6 downto 5) = "00" else
           '0' ;

process (clkbd, rst)				-- read and check port D
begin
if rst = '1' then
	datad <= "0000000000000000" ;
	oldd <= "0000000000000000" ;
	addrd <= "10000000" ;
	matchcd <= '0' ;
	en_d <= '0' ;
	rstd <= '1' ;
	we_d <= '0' ;
elsif clkbd'event and clkbd = '1' then
	rstd <= '0' ;
	if en_d = '1' then
		datad <= dod ;
		oldd <= datad ;
	end if ;
	if emptycd = '1' then
		en_d <= '1' ;
		addrd(6 downto 0) <= addrd(6 downto 0) + 1 ;
	else
		en_d <= '0' ;
	end if ;
	if datad = oldd - 1 then
		matchcd <= '1' ;
	else
		matchcd <= '0' ;
	end if ;
end if ;
end process ;

end arch_top_ve;


